from datetime import datetime
from pydantic import BaseModel, EmailStr, NaiveDatetime

from bs4 import BeautifulSoup


class MiweoCustomerAddress(BaseModel):
    address: list[str]
    city: str
    zip_code: int
    country: str


class MiweoSubscription(BaseModel):
    ref: str
    offer: str
    status: str
    activation: NaiveDatetime
    address: str
    oi: str
    actions: str


class MiweoCustomer(BaseModel):
    name: str
    email: EmailStr
    creation_date: NaiveDatetime
    address: MiweoCustomerAddress
    subsciption: MiweoSubscription


def parse_customer_info(content: str) -> MiweoCustomer:
    soup = BeautifulSoup(content, "html.parser")
    customer_info_element = soup.find("div", attrs={"id": "knetCustomerReviewCard"})
    customer_name = customer_info_element.select_one(".title").text  # type: ignore
    information_items = customer_info_element.find_all(  # type: ignore
        "div", attrs={"class": "informationItem"}
    )
    phones = []
    email = None
    creation_date = None
    for ii in information_items:
        if ii.select_one(".fa-envelope"):  # type: ignore
            email = ii.select_one(".text").text  # type: ignore
            continue
        if ii.select_one(".fa-phone-alt"):  # type: ignore
            phones.append(ii.select_one(".text").text)  # type: ignore
            continue
        if (div_label := ii.select_one("div.label")) is not None and div_label.text == "Date de création":  # type: ignore
            creation_date_str = ii.select_one("div.text").text  # type: ignore
            creation_date = datetime.strptime(creation_date_str, "%d/%m/%Y à %H:%M:%S")
            continue
    address_content = customer_info_element.select_one(".addressContent")  # type: ignore
    address = [ee.text for ee in address_content.select("div .row")[:2]]  # type: ignore
    city = address_content.select_one(".city").text  # type: ignore
    zip_code = int(address_content.select_one(".zipCode").text)  # type: ignore
    country = address_content.find_all()[-1].text  # type: ignore
    review_card_widgets = soup.select("div.reviewCardWidget")
    subscription_elem = review_card_widgets[1]
    subscription_dict = dict(
        zip(
            [ee.text.strip() for ee in subscription_elem.select("th")],
            [ee.text.strip() for ee in subscription_elem.select("tr td.cell")],
        )
    )
    assert email is not None
    assert creation_date is not None
    customer = MiweoCustomer(
        name=customer_name,
        email=email,
        creation_date=creation_date,
        address=MiweoCustomerAddress(
            address=address, city=city, zip_code=zip_code, country=country
        ),
        subsciption=MiweoSubscription(
            ref=subscription_dict["Référence"],
            offer=subscription_dict["Offre"],
            status=subscription_dict["Statut"],
            activation=datetime.strptime(subscription_dict["Activation"], "%d/%m/%Y"),
            address=subscription_dict["Adresse"],
            oi=subscription_dict["OI"],
            actions=subscription_dict["Actions"],
        ),
    )
    return customer


if __name__ == "__main__":
    with open("data/rec.html") as f:
        result = parse_customer_info(f.read())
    print(result)
