from typing import Type, Tuple
from pathlib import Path
from xdg import BaseDirectory

from pydantic import BaseModel, HttpUrl
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
    YamlConfigSettingsSource,
)


# from hatchet_sdk import ClientConfig as HatchetClientConfig
#
# Minimal Mock hatchet config as it is amazingly slow to load.
# The real instanciation, using the settings defined here,
# is done in hatchet_client.py


class HatchetClientTlsConfig(BaseSettings):
    strategy: str = "none"


class HatchetClientLogConfig(BaseSettings):
    level: int = 40


class HatchetClientConfig(BaseSettings):
    model_config = SettingsConfigDict(
        env_prefix="HATCHET_CLIENT_",
    )

    token: str = ""
    tls: HatchetClientTlsConfig = HatchetClientTlsConfig()
    log: HatchetClientLogConfig = HatchetClientLogConfig()


config_files = [
    Path(BaseDirectory.xdg_config_home) / "knoc" / "conf.yaml",
    Path(Path.cwd().root) / "etc" / "knoc" / f"conf.yml",
]


class KSplynxSettings(BaseModel):
    url: HttpUrl = HttpUrl("https://gp-acs.k-net.fr")
    key: str = ""
    sec: str = ""
    db_schema: str = "splynx"
    db_table_customers: str = "customers"
    authtypeapikey: str = "api_key"


class HatchetSettings(BaseModel):
    token: str = ""
    tlsstrategy: str = "none"


class KMiweoSettings(BaseModel):
    url: HttpUrl = HttpUrl("https://knet.miweo.com")
    cookie: str = ""
    phpsessid: str = ""
    db_table_records_list_prefix: str = "records_"
    db_table_records_list: str = "records"
    db_schema: str = "miweo"


class DBSettings(BaseModel):
    # url: str = "postgresql+psycopg://knoc:knoc@knoc-db:5432/knoc"
    username: str = "knoc"
    password: str = "knoc"
    host: str = "knoc-db"
    port: int = 5432
    database: str = "knoc"
    echo: bool = False


class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        nested_model_default_partial_update=True,
        env_nested_delimiter="_",
        env_file=(".env", ".env.prod"),
    )

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        configs = [
            YamlConfigSettingsSource(settings_cls, yaml_file=cf) for cf in config_files
        ]
        return (
            env_settings,
            init_settings,
            dotenv_settings,
            file_secret_settings,
            *configs,
        )

    miweo: KMiweoSettings = KMiweoSettings()
    splynx: KSplynxSettings = KSplynxSettings()
    hatchet_client: HatchetClientConfig = HatchetClientConfig()
    hatchet_client_debug: bool = False
    db: DBSettings = DBSettings()

    data_path: Path = Path("data")


settings = Settings()
